"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const dgram_1 = __importDefault(require("dgram"));
const events_1 = __importDefault(require("events"));
const net_1 = __importDefault(require("net"));
const tplink_smarthome_crypto_1 = require("tplink-smarthome-crypto");
const udp_server_1 = __importDefault(require("./udp-server"));
const log = (0, debug_1.default)('device');
const logUdp = (0, debug_1.default)('device:udp');
const logUdpErr = (0, debug_1.default)('device:udp:error');
const logTcp = (0, debug_1.default)('device:tcp');
const logTcpErr = (0, debug_1.default)('device:tcp:error');
const delay = (t) => new Promise((resolve) => {
    setTimeout(resolve, t);
});
class DeviceNetworking extends events_1.default {
    constructor({ device, model, port = 0, address = '0.0.0.0', responseDelay = 0, }) {
        super();
        this.address = '0.0.0.0';
        this.port = 0;
        this.responseDelay = 0;
        this.serverBound = false;
        this.udpSocketBound = false;
        this.device = device;
        this.model = model;
        this.port = port;
        this.address = address;
        this.responseDelay = responseDelay;
        this.serverBound = false;
        this.udpSocketBound = false;
    }
    processUdpMessage(msg, rinfo) {
        const decryptedMsg = (0, tplink_smarthome_crypto_1.decrypt)(msg).toString('utf8');
        logUdp('[%s] UDP receiving', this.model, rinfo.port, rinfo.address);
        this.emit('data', {
            time: Date.now(),
            protocol: 'udp',
            message: decryptedMsg,
            remoteAddress: rinfo.address,
            remotePort: rinfo.port,
        });
        logUdp(decryptedMsg);
        let response;
        let responseForLog;
        try {
            ({ response, responseForLog } =
                this.device.processUdpMessage(decryptedMsg));
        }
        catch (err) {
            logUdpErr('processUdpMessage, could not process:', decryptedMsg);
            logUdpErr(err);
            this.emit('error', err);
            return;
        }
        if (response !== undefined) {
            delay(this.responseDelay).then(() => {
                if (this.udpSocket == null)
                    throw new Error('udpSocket was undefined');
                if (response == null)
                    throw new Error('Response was undefined');
                logUdp('[%s] UDP responding, delay:%s,', this.model, this.responseDelay, rinfo.port, rinfo.address);
                logUdp(responseForLog);
                this.emit('response', {
                    time: Date.now(),
                    protocol: 'tcp',
                    message: responseForLog,
                    remoteAddress: rinfo.address,
                    remotePort: rinfo.port,
                });
                this.udpSocket.send(response, 0, response.length, rinfo.port, rinfo.address);
            });
        }
    }
    processTcpMessage(msg, socket) {
        logTcp('[%s] TCP DATA', this.model, socket.remoteAddress, socket.remotePort);
        const decryptedMsg = (0, tplink_smarthome_crypto_1.decryptWithHeader)(msg).toString('utf8');
        logTcp(decryptedMsg);
        this.emit('data', {
            time: Date.now(),
            protocol: 'tcp',
            message: decryptedMsg,
            localAddress: socket.localAddress,
            localPort: socket.localPort,
            remoteAddress: socket.remoteAddress,
            remotePort: socket.remotePort,
        });
        let response;
        let responseForLog;
        try {
            ({ response, responseForLog } =
                this.device.processTcpMessage(decryptedMsg));
        }
        catch (err) {
            logTcpErr('processTcpMessage, could not process:', decryptedMsg);
            logTcpErr(err);
            this.emit('error', err);
            return;
        }
        if (response !== undefined) {
            delay(this.responseDelay).then(() => {
                if (response == null)
                    throw new Error('Response was undefined');
                logTcp('[%s] TCP responding, delay:%s,', this.model, this.responseDelay, socket.address());
                logTcp(responseForLog);
                this.emit('response', {
                    time: Date.now(),
                    protocol: 'tcp',
                    message: responseForLog,
                    localAddress: socket.localAddress,
                    localPort: socket.localPort,
                    remoteAddress: socket.remoteAddress,
                    remotePort: socket.remotePort,
                });
                socket.write(response);
                if (this.device.endSocketAfterResponse) {
                    socket.end();
                }
            });
        }
    }
    async start() {
        return new Promise((resolve, reject) => {
            try {
                let udpAddress;
                let retryCount = 0;
                const server = net_1.default.createServer((socket) => {
                    socket.on('data', (chunk) => {
                        this.processTcpMessage(chunk, socket);
                    });
                    socket.on('error', (err) => {
                        this.emit('error', err);
                    });
                    socket.on('end', () => {
                        socket.end();
                    });
                });
                this.server = server;
                server.on('listening', () => {
                    this.serverBound = true;
                    const tcpAddress = server.address();
                    if (tcpAddress == null)
                        throw new Error('Server.address() returned null');
                    this.address = tcpAddress.address;
                    this.port = tcpAddress.port;
                    log('[%s] TCP server bound', this.model, tcpAddress);
                    resolve(this);
                });
                server.on('error', (err) => {
                    logTcpErr(err);
                    if ('code' in err &&
                        // @ts-ignore: it's fine
                        (err === null || err === void 0 ? void 0 : err.code) === 'EADDRINUSE' &&
                        udpAddress != null &&
                        retryCount < 2) {
                        retryCount += 1;
                        logTcpErr('Address in use, retrying...');
                        delay(500).then(() => {
                            server.close();
                            if (udpAddress == null)
                                throw new Error('udpAddress was undefined');
                            server.listen({ port: udpAddress.port, host: this.address });
                        });
                    }
                    reject(err);
                });
                // bind to UDP then TCP and share port
                const udpSocket = dgram_1.default.createSocket({ type: 'udp4', reuseAddr: true });
                this.udpSocket = udpSocket;
                // Save to call removeListener on #stop()
                this.udpCb = (msg, rinfo) => {
                    this.processUdpMessage(msg, rinfo);
                };
                udp_server_1.default.on('message', this.udpCb);
                udpSocket.on('message', this.udpCb);
                udpSocket.on('error', (exception) => {
                    logUdpErr(exception);
                    udpSocket.close();
                    this.udpSocketBound = false;
                    reject(exception);
                });
                udpSocket.bind(this.port, this.address, () => {
                    this.udpSocketBound = true;
                    udpAddress = udpSocket.address();
                    log('[%s] UDP bound', this.model, udpAddress);
                    server.listen({ port: udpAddress.port, host: this.address });
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    async stop() {
        return new Promise((resolve) => {
            if (typeof this.udpCb === 'function') {
                udp_server_1.default.removeListener('message', this.udpCb);
            }
            if (this.udpSocketBound) {
                if (this.udpSocket != null)
                    this.udpSocket.close();
                this.udpSocketBound = false;
            }
            if (this.serverBound) {
                if (this.server != null)
                    this.server.close(() => {
                        this.serverBound = false;
                        resolve(this);
                    });
            }
            else {
                resolve(this);
            }
        });
    }
}
exports.default = DeviceNetworking;
//# sourceMappingURL=device-networking.js.map